// Copyright (c) 1997 Andrew Maltsev <am@amsoft.ru>
// All rights reserved
// 
// Stubs for use in plain C programs
//
#include <stdio.h>
#include "es_int.h"

///////////////////////////////////////////////////////////////////////////////
// Creating driver (opening database)
//
esqlDB
easysql_open(char const *dbname)
{ return (esqlDB)new easysql(dbname);
}

///////////////////////////////////////////////////////////////////////////////
// Destroying driver (closing database)
//
int
easysql_close(esqlDB drv)
{ if(!drv)
   { log(D_ERROR|D_C,"Attempt to destroy non-created driver");
     return false;
   }
  delete (easysql *)drv;
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Text of last error
//
char const *
easysql_error_text(esqlDB drv)
{ if(!drv)
   { log(D_ERROR|D_C,"Attempt to get error text from non-created driver");
     return "Driver was not created (earlest phase error)";
   }
  return ((easysql *)drv)->error_text();
}

///////////////////////////////////////////////////////////////////////////////
// Sending query and fetching results
//
esqlResult
easysql_query(esqlDB drv, char const *q, ...)
{ va_list ap;
  va_start(ap,q);
  esqlResult r=easysql_query_ap(drv,q,ap);
  va_end(ap);
  return r;
}

///////////////////////////////////////////////////////////////////////////////
// Sending query from va_list
//
esqlResult
easysql_query_ap(esqlDB drv, char const *q, va_list ap)
{ if(!drv)
   { log(D_ERROR|D_C,"Attempt to send query to non-created driver");
     return NULL;
   }
  return ((easysql *)drv)->_query(q,ap);
}

///////////////////////////////////////////////////////////////////////////////
// Fetching next row in sequence of results
//
esqlRow
easysql_row(esqlDB drv, esqlResult r)
{ if(!drv)
   { log(D_ERROR|D_C,"Attempt to fetch row from non-created driver");
     return NULL;
   }
  return ((easysql *)drv)->row(r);
}

///////////////////////////////////////////////////////////////////////////////
// Freeing results of query
//
int
easysql_rfree(esqlDB drv, esqlResult r)
{ if(!drv)
   { log(D_ERROR|D_C,"Attempt to free results referenced from non-created driver");
     return false;
   }
  return ((easysql *)drv)->rfree(r);
}

///////////////////////////////////////////////////////////////////////////////
// Resetting all driver's options and connection
//
void
easysql_reset(esqlDB drv)
{ if(!drv)
   { log(D_ERROR|D_C,"Attempt to reset non-created driver");
     return;
   }
  ((easysql *)drv)->reset();
}

///////////////////////////////////////////////////////////////////////////////
// Setting option by name and value (like setenv)
//
int
easysql_set_option(esqlDB drv, char const *name, char const *value)
{ if(!drv)
   { log(D_ERROR|D_C,"Attempt to set option(\"%s\",\"%s\") for non-created driver",name,value);
     return false;
   }
  return ((easysql *)drv)->option(name,value);
}

///////////////////////////////////////////////////////////////////////////////
// Putting option by "name=value" string (like putenv)
//
int
easysql_put_option(esqlDB drv, char const *option)
{ if(!drv)
   { log(D_ERROR|D_C,"Attempt to set option(\"%s\") for non-created driver",option);
     return false;
   }
  return ((easysql *)drv)->option(option);
}

///////////////////////////////////////////////////////////////////////////////
// Checking driver's internal structures
//
int
easysql_ok(esqlDB drv)
{ if(!drv)
   { log(D_ERROR|D_C,"Attempt to check non-created driver");
     return false;
   }
  return bool(*((easysql *)drv));
}

///////////////////////////////////////////////////////////////////////////////
// Checking internal real engine driver
//
int
easysql_driver_ok(esqlDB drv)
{ if(!drv)
   { log(D_ERROR|D_C,"Attempt to check driver of non-created driver");
     return false;
   }
  return ((easysql *)drv)->driver_ok();
}

///////////////////////////////////////////////////////////////////////////////
// Fetching driver status
//
char const *
easysql_driver_status(esqlDB drv)
{ if(!drv)
   { log(D_ERROR|D_C,"Attempt to fetch status of non-created driver");
     return "Driver was not created";
   }
  return ((easysql *)drv)->driver_status();
}

///////////////////////////////////////////////////////////////////////////////
// Fetching driver version
//
char const *
easysql_driver_version(esqlDB drv)
{ if(!drv)
   { log(D_ERROR|D_C,"Attempt to fetch version of non-created driver");
     return "null " EASYSQL_VERSION;
   }
  return ((easysql *)drv)->driver_version();
}

///////////////////////////////////////////////////////////////////////////////
// Setting retry delay
//
unsigned
easysql_set_retry_delay(esqlDB drv, unsigned d)
{ if(!drv)
   { log(D_ERROR|D_C,"Attempt to set retry delay for non-created driver");
     return 0;
   }
  return ((easysql *)drv)->set_retry_delay(d);
}


///////////////////////////////////////////////////////////////////////////////
// Converting string to SQL style
//
char const *
easysql_string(esqlDB drv, char const *str, char *buf)
{ if(!drv)
   { log(D_ERROR|D_C,"Attempt to convert \"%s\" by means of non-created driver",str);
     return str;
   }
  return ((easysql *)drv)->string(str,buf);
}

//////////////////////////////////////////////////////////////////////////////
R("$Id: c_stubs.C,v 1.2 1997/10/23 18:40:32 am Exp $");
