// Copyright (c) 1997 Andrew Maltsev <am@amsoft.ru>
// All rights reserved
// 
// A space to store all arguments to driver (host, database, user-name,
// password and so on)
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "es_int.h"

///////////////////////////////////////////////////////////////////////////////
// Constructing an empty storage and filling it with initial parameters
// if any
//
easysql_arguments::easysql_arguments(char const *a)
                  :values(NULL)
                  ,names(NULL)
                  ,num(0)
{ if(a) merge(a);
}

///////////////////////////////////////////////////////////////////////////////
// Destructing and freeing occupied memory
//
easysql_arguments::~easysql_arguments()
{ reset();
}

///////////////////////////////////////////////////////////////////////////////
// Freeing occupied memory
//
void
easysql_arguments::reset()
{ for(unsigned i=0; i!=num; i++)
   { Free(names[i]);
     Free(values[i]);
   }
  Free(values);
  Free(names);
  values=NULL;
  names=NULL;
  num=0;
}

///////////////////////////////////////////////////////////////////////////////
// Splitting string of a form "a=b c=d e=f ..."
//
bool
easysql_arguments::merge(char const *a)
{ if(!a || !*a)
   return false;

  log(D_MISC|D_L4,"arguments(\"%s\")",a);

  char *args=strDup(a);
  char *name=args;
  while(*name)
   { while(isspace(*name)) name++;
     if(!*name) break;
     char *value=name;
     while(*value && *value!='=') value++;
     if(!*value)
      { log(D_ERROR|D_MISC,"Syntax error at args=`%s'",a);
        break;
      }
     *value++=0;
     while(isspace(*value)) value++;
     char *s=value;
     while(*s && !isspace(*s)) s++;
     if(*s) *s++=0;
     merge(name,value);
     name=s;
   }
  Free(args);
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Setting name/value pair. Name and/or value may be NULLs - it is ignored
// with `true' result.
//
bool
easysql_arguments::merge(char const *name, char const *value)
{ if(!name || !*name || !value)
   return true;

  log(D_DRV|D_L4,"arguments(\"%s\",\"%s\")",name,value);
  for(unsigned i=0; i!=num; i++)
   if(!strcasecmp(names[i],name))
    { Free(values[i]);
      values[i]=strDup(value);
      return true;
    }

  if(num%10==0)
   { names=(typeof(names))reAlloc(names,sizeof(*names)*(num+10));
     values=(typeof(values))reAlloc(values,sizeof(*values)*(num+10));
   }
  names[num]=strDup(name);
  values[num]=strDup(value);
  num++;
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Selecting value by name. This method made static because it is
// transferred to shared object and our goal is to exchange with object
// only pointers.
//
char const *
easysql_arguments::value(easysql_arguments const *that, char const *name)
{ for(unsigned i=0; i!=that->num; i++)
   if(!strcmp(that->names[i],name))
    return that->values[i];
  return NULL;
}

///////////////////////////////////////////////////////////////////////////////
R("$Id: es_args.C,v 1.6 1997/09/20 21:54:27 am Exp $");
