// Copyright (c) 1997 Andrew Maltsev <am@amsoft.ru>
// All rights reserved
// 
// Logging.
//
#include <string.h>
#include <time.h>
#include "es_int.h"

///////////////////////////////////////////////////////////////////////////////
// Default log level
//
static unsigned logLevel=D_ERROR;

///////////////////////////////////////////////////////////////////////////////
// Log file. If NULL (by default) stderr is used instead
//
static FILE *logFile=NULL;
static char *logName=NULL;

///////////////////////////////////////////////////////////////////////////////
// Writing message to log-file or stderr
//
void
easysql_log(unsigned level, char const *fmt, ...)
{ if(logLevel&level)
   { va_list ap;
     va_start(ap,fmt);
     FILE *f=stderr;
     if(logFile)
      { static unsigned lines=0;
        if(lines++>1000)		// Doing close/open every 1000 lines (at least)
         easysql_reopen_log();
        time_t t=time(NULL);
        struct tm *tm=localtime(&t);
        char const *mon[]={"Jan","Feb","Mar","Apr","May","Jun"
                          ,"Jul","Aug","Sep","Oct","Nov","Dec"};
        fprintf(logFile,"%02u-%s-%04u %02u:%02u:%02u "
                       ,tm->tm_mday,mon[tm->tm_mon],tm->tm_year+1900
                       ,tm->tm_hour,tm->tm_min,tm->tm_sec);
        f=logFile;
      }
     if(level&D_ERROR)
      fprintf(f,"@ ");
     else if(level&D_L1)
      fprintf(f,"* ");
     else if(level&D_L2)
      fprintf(f,"+ ");
     else if(level&D_L3)
      fprintf(f,"- ");
     else if(level&D_L4)
      fprintf(f,". ");
     vfprintf(f,fmt,ap);
     va_end(ap);
     if(fmt[strlen(fmt)-1]!='\n')
      fputc('\n',f);
   }
}

///////////////////////////////////////////////////////////////////////////////
// Setting log-level
//
void
easysql_set_log_level(unsigned l)
{ logLevel=l;
}

///////////////////////////////////////////////////////////////////////////////
// Checking log level
//
unsigned
easysql_get_log_level()
{ return logLevel;
}

///////////////////////////////////////////////////////////////////////////////
// Setting log file
//
int
easysql_set_log_file(char const *n)
{ if(logName)
   { Free(logName);
     logName=NULL;
   }
  if(n && !strcasecmp(n,"stderr"))
   n=NULL;
  if(n && !strcasecmp(n,"none"))
   { n=NULL;
     logLevel=D_NONE;
   }
  if(n)
   logName=strDup(n);
  easysql_reopen_log();
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Checking log file
//
char const *
easysql_get_log_file()
{ return logName ? logName : (logLevel == D_NONE ? "none" : "stderr");
}

///////////////////////////////////////////////////////////////////////////////
// Reopening log file. May be called from user program as a responce to
// SIGHUP (for example) and is called automatically before logging every
// 1000-th record.
//
void
easysql_reopen_log()
{ if(logFile)
   { fclose(logFile);
     logFile=NULL;
   }
  if(logName)
   logFile=fopen(logName,"a");
}

///////////////////////////////////////////////////////////////////////////////
R("$Id: log.C,v 1.2 1997/10/23 18:40:41 am Exp $");
